<?php
/**
 * WHMCS Sample Payment Callback File
 *
 * This sample file demonstrates how a payment gateway callback should be
 * handled within WHMCS.
 *
 * It demonstrates verifying that the payment gateway module is active,
 * validating an Invoice ID, checking for the existence of a Transaction ID,
 * Logging the Transaction for debugging and Adding Payment to an Invoice.
 *
 * For more information, please refer to the online documentation.
 *
 * @see https://developers.whmcs.com/payment-gateways/callbacks/
 *
 * @copyright Copyright (c) WHMCS Limited 2017
 * @license http://www.whmcs.com/license/ WHMCS Eula
 */

// Require libraries needed for gateway module functions.
require_once __DIR__ . '/../../../init.php';
require_once __DIR__ . '/../../../includes/gatewayfunctions.php';
require_once __DIR__ . '/../../../includes/invoicefunctions.php';

// Detect module name from filename.
$gatewayModuleName = basename(__FILE__, '.php');

// Fetch gateway configuration parameters.
$gatewayParams = getGatewayVariables($gatewayModuleName);

// Die if module is not active.
if (!$gatewayParams['type']) {
    die("Module Not Activated");
}

$invoice = explode('WHMCS', $_POST['ConversationId']);

$response = teqpay_checkPayment($_POST['OrderId']);
// echo $response; exit;

// Retrieve data returned in payment gateway callback
// Varies per payment gateway
$success = $response;
$invoiceId = current($invoice);
$transactionId = $_POST["OrderId"];
$paymentAmount = $response ? $_POST['Details']['TotalPrice'] : 0;
$paymentFee = 0;


$transactionStatus = $success ? 'Success' : 'Failure';

/**
 * Validate callback authenticity.
 *
 * Most payment gateways provide a method of verifying that a callback
 * originated from them. In the case of our example here, this is achieved by
 * way of a shared secret which is used to build and compare a hash.
 */
// $secretKey = $gatewayParams['secretKey'];
// if ($hash != md5($invoiceId . $transactionId . $paymentAmount . $secretKey)) {
//     $transactionStatus = 'Hash Verification Failure';
//     $success = false;
// }

/**
 * Validate Callback Invoice ID.
 *
 * Checks invoice ID is a valid invoice number. Note it will count an
 * invoice in any status as valid.
 *
 * Performs a die upon encountering an invalid Invoice ID.
 *
 * Returns a normalised invoice ID.
 *
 * @param int $invoiceId Invoice ID
 * @param string $gatewayName Gateway Name
 */
$invoiceId = checkCbInvoiceID($invoiceId, $gatewayParams['name']);

/**
 * Check Callback Transaction ID.
 *
 * Performs a check for any existing transactions with the same given
 * transaction number.
 *
 * Performs a die upon encountering a duplicate.
 *
 * @param string $transactionId Unique Transaction ID
 */
checkCbTransID($transactionId);

/**
 * Log Transaction.
 *
 * Add an entry to the Gateway Log for debugging purposes.
 *
 * The debug data can be a string or an array. In the case of an
 * array it will be
 *
 * @param string $gatewayName        Display label
 * @param string|array $debugData    Data to log
 * @param string $transactionStatus  Status
 */
logTransaction($gatewayParams['name'], $_POST, $transactionStatus);
$paymentSuccess = false;
if ($success) {

    /**
     * Add Invoice Payment.
     *
     * Applies a payment transaction entry to the given invoice ID.
     *
     * @param int $invoiceId         Invoice ID
     * @param string $transactionId  Transaction ID
     * @param float $paymentAmount   Amount paid (defaults to full balance)
     * @param float $paymentFee      Payment fee (optional)
     * @param string $gatewayModule  Gateway module name
     */
    addInvoicePayment(
        $invoiceId,
        $transactionId,
        $paymentAmount,
        $paymentFee,
        $gatewayModuleName
    );
    $paymentSuccess = true;
}


function teqpay_checkPayment($order_id)
    {
        $gatewayModuleName = basename(__FILE__, '.php');

        // Fetch gateway configuration parameters.
        $gatewayParams = getGatewayVariables($gatewayModuleName);

       
        $sandbox = $gatewayParams['testMode']; 
        $url = $sandbox == 'on' ? 'https://merchant-test.teqpay.com/api/teqpay/paymentcheck' : 'https://merchant.teqpay.com/api/teqpay/paymentcheck';


        $post = [
            "orderId" => $order_id,
            "language" => 'TR'
        ];

        $json = json_encode($post);

        $response = teqpayreturn_curl($url, $json);

        if($response['Details']['OrderStatusCode'] == 1){
            return true;
        }

        return false;
    }

    function teqpayreturn_curl($url, $json)
    {
        $gatewayModuleName = basename(__FILE__, '.php');
        $gatewayParams = getGatewayVariables($gatewayModuleName);

        $secret_key = $gatewayParams['apiKey'];
        $api_key = $gatewayParams['secretKey'];

        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_VERBOSE => true,
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => $json,
            CURLOPT_HTTPHEADER => [
              "Content-Type: application/json",
              "ApiKey: {$api_key}",
              "SecretKey: {$secret_key}",
            ]
        ]);

        $response = curl_exec($curl);
		$response = json_decode($response, true);
        curl_close($curl);

        return $response;
    }

    /**
 * Redirect to invoice.
 *
 * Performs redirect back to the invoice upon completion of the 3D Secure
 * process displaying the transaction result along with the invoice.
 *
 * @param int $invoiceId        Invoice ID
 * @param bool $paymentSuccess  Payment status
 */
callback3DSecureRedirect($invoiceId, $paymentSuccess);