<?php

use WHMCS\User\Alert;

/**
 * WHMCS Sample Payment Gateway Module
 *
 * Payment Gateway modules allow you to integrate payment solutions with the
 * WHMCS platform.
 *
 * This sample file demonstrates how a payment gateway module for WHMCS should
 * be structured and all supported functionality it can contain.
 *
 * Within the module itself, all functions must be prefixed with the module
 * filename, followed by an underscore, and then the function name. For this
 * example file, the filename is "teqpay" and therefore all functions
 * begin "teqpay_".
 *
 * If your module or third party API does not support a given function, you
 * should not define that function within your module. Only the _config
 * function is required.
 *
 * For more information, please refer to the online documentation.
 *
 * @see https://developers.whmcs.com/payment-gateways/
 *
 * @copyright Copyright (c) WHMCS Limited 2017
 * @license http://www.whmcs.com/license/ WHMCS Eula
 */

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

/**
 * Define module related meta data.
 *
 * Values returned here are used to determine module related capabilities and
 * settings.
 *
 * @see https://developers.whmcs.com/payment-gateways/meta-data-params/
 *
 * @return array
 */
function teqpay_MetaData()
{
    return array(
        'DisplayName' => 'Teqpay Gateway Module',
        'APIVersion' => '1.1', // Use API Version 1.1
        'DisableLocalCreditCardInput' => true,
        'TokenisedStorage' => false,
    );
}

/**
 * Define gateway configuration options.
 *
 * The fields you define here determine the configuration options that are
 * presented to administrator users when activating and configuring your
 * payment gateway module for use.
 *
 * Supported field types include:
 * * text
 * * password
 * * yesno
 * * dropdown
 * * radio
 * * textarea
 *
 * Examples of each field type and their possible configuration parameters are
 * provided in the sample function below.
 *
 * @return array
 */
function teqpay_config()
{
    return array(
        // the friendly display name for a payment gateway should be
        // defined here for backwards compatibility
        'FriendlyName' => array(
            'Type' => 'System',
            'Value' => 'Teqpay Sanapos',
        ),
        // a text field type allows for single line text input
        'apiKey' => array(
            'FriendlyName' => 'Api Key',
            'Type' => 'text',
            'Size' => '50',
            'Default' => '',
            'Description' => 'Enter your api key here',
        ),
        // a password field type allows for masked text input
        'secretKey' => array(
            'FriendlyName' => 'Secret Key',
            'Type' => 'password',
            'Size' => '50',
            'Default' => '',
            'Description' => 'Enter secret key here',
        ),
        // the yesno field type displays a single checkbox option
        'testMode' => array(
            'FriendlyName' => 'Test Mode',
            'Type' => 'yesno',
            'Description' => 'Tick to enable test mode',
        ),

    );
}

/**
 * Payment link.
 *
 * Required by third party payment gateway modules only.
 *
 * Defines the HTML output displayed on an invoice. Typically consists of an
 * HTML form that will take the user to the payment gateway endpoint.
 *
 * @param array $params Payment Gateway Module Parameters
 *
 * @see https://developers.whmcs.com/payment-gateways/third-party-gateway/
 *
 * @return string
 */
function teqpay_link($params)
{
    // Gateway Configuration Parameters
    $api_key = $params['apiKey'];
    $secret_key = $params['secretKey'];
    $testMode = $params['testMode'];
    $items = [];
    //error_log(print_r($params['cart'], true));
    foreach ($params['cart']->items as $item) {
        $items[] = [
            'merchantItemId' => $item->id,
            'itemType' => 'Ürün',
            'itemCategory' => 'Ürün',
            'itemName' => $item->name,
            'itemPrice' => $params['amount'],
            'itemQuantity' => (int) $item->qty,
        ];
    }

    $systemUrl = $params['systemurl'];
    $billing = [
        "billingName" => $params['clientdetails']['firstname'] . ' ' . $params['clientdetails']['lastname'],
        "billingCity" => $params['clientdetails']['city'],
        "billingCountry" => 'Türkiye',
        "billingAddress" => $params['clientdetails']['address1'],
    ];

    $shipping = [
        "shippingContactName" => $params['clientdetails']['firstname'] . ' ' . $params['clientdetails']['lastname'],
        "shippingCity" => $params['clientdetails']['city'],
        "shippingCountry" => 'Türkiye',
        "shippingAddress" => $params['clientdetails']['address1'],
    ];

    $data = [

        // Customer
        'customerName' => $params['clientdetails']['firstname'] . ' ' . $params['clientdetails']['lastname'],
        'customerIpAddress' => teqpay_ip(),
        'customerCitizenNo' => 12345678901,
        'customerEmail' => $params['clientdetails']['email'],
        'customerPhone' => $params['clientdetails']['phonenumber'],
        'language' => 'TR',
        // Order
        'price' => number_format(abs($params['amount']), 2, ".", ""),
        'currency' => $params['currency'],

        'conversationId' => $params['invoiceid'] . 'WHMCS' . md5(microtime()),

        // Products
        'products' => $items,

        // Billign
        'billing' => $billing,

        // Shipping
        'shipping' => $shipping,

        'callbackUrl' => $systemUrl . 'modules/gateways/callback/' . $moduleName . '.php',
        
    ];
    
     
    $url = $testMode == 'on' ? 'https://merchant-test.teqpay.com/api/teqpay/paywithcommonpage' : 'https://merchant.teqpay.com/api/teqpay/paywithcommonpage';

    $response = teqpay_curl($data, $api_key, $secret_key, $url);

    if($response['Result'] == true){
        echo ("<script>location.href='" . $response['PaymentData']['PaymentUrl'] . "'</script>");
        exit;
    } 

}

function teqpay_ip()
	{
		$ip = '';
		if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
			$ip = $_SERVER['HTTP_CLIENT_IP'];
		} elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
		} else {
			$ip = $_SERVER['REMOTE_ADDR'];
		}

		return $ip;
	}

function teqpay_curl($data, $api_key, $secret_key, $url)
{
    $curl = curl_init();
    curl_setopt_array($curl, [
        CURLOPT_VERBOSE => true,
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS => json_encode($data),
        CURLOPT_HTTPHEADER => [
            "Content-Type: application/json",
            "ApiKey: {$api_key}",
            "SecretKey: {$secret_key}",
        ]
    ]);

    $response = curl_exec($curl);
    $response = json_decode($response, true);
    curl_close($curl);

    return $response;
}

