<?php
/**
 * WHMCS Sample Merchant Gateway Module
 *
 * This sample file demonstrates how a merchant gateway module supporting
 * 3D Secure Authentication, Captures and Refunds can be structured.
 *
 * If your merchant gateway does not support 3D Secure Authentication, you can
 * simply omit that function and the callback file from your own module.
 *
 * Within the module itself, all functions must be prefixed with the module
 * filename, followed by an underscore, and then the function name. For this
 * example file, the filename is "teqpaycard" and therefore all functions
 * begin "teqpaycard_".
 *
 * For more information, please refer to the online documentation.
 *
 * @see https://developers.whmcs.com/payment-gateways/
 *
 * @copyright Copyright (c) WHMCS Limited 2019
 * @license http://www.whmcs.com/license/ WHMCS Eula
 */

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

/**
 * Define module related meta data.
 *
 * Values returned here are used to determine module related capabilities and
 * settings.
 *
 * @see https://developers.whmcs.com/payment-gateways/meta-data-params/
 *
 * @return array
 */
function teqpaycard_MetaData()
{
    return array(
        'DisplayName' => 'Teqpay Credit Card',
        'APIVersion' => '1.1', // Use API Version 1.1
    );
}

/**
 * Define gateway configuration options.
 *
 * The fields you define here determine the configuration options that are
 * presented to administrator users when activating and configuring your
 * payment gateway module for use.
 *
 * Supported field types include:
 * * text
 * * password
 * * yesno
 * * dropdown
 * * radio
 * * textarea
 *
 * Examples of each field type and their possible configuration parameters are
 * provided in the sample function below.
 *
 * @see https://developers.whmcs.com/payment-gateways/configuration/
 *
 * @return array
 */
function teqpaycard_config()
{
    return array(
        // the friendly display name for a payment gateway should be
        // defined here for backwards compatibility
        'FriendlyName' => array(
            'Type' => 'System',
            'Value' => 'Teqpay Credit Card',
        ),
        // a text field type allows for single line text input
        'apiKey' => array(
            'FriendlyName' => 'Api Key',
            'Type' => 'text',
            'Size' => '25',
            'Default' => '',
            'Description' => 'Enter your Api Key here',
        ),
        // a password field type allows for masked text input
        'secretKey' => array(
            'FriendlyName' => 'Secret Key',
            'Type' => 'password',
            'Size' => '25',
            'Default' => '',
            'Description' => 'Enter secret key here',
        ),
        // the yesno field type displays a single checkbox option
        'testMode' => array(
            'FriendlyName' => 'Test Mode',
            'Type' => 'yesno',
            'Description' => 'Tick to enable test mode',
        ),
        
    );
}


/**
 * Capture payment.
 *
 * Called when a payment is to be processed and captured.
 *
 * The card cvv number will only be present for the initial card holder present
 * transactions. Automated recurring capture attempts will not provide it.
 *
 * @param array $params Payment Gateway Module Parameters
 *
 * @see https://developers.whmcs.com/payment-gateways/merchant-gateway/
 *
 * @return array Transaction response status
 */
function teqpaycard_capture($params)
{
    // Gateway Configuration Parameters
    $api_key = $params['apiKey'];
    $secret_key = $params['secretKey'];
    $testMode = $params['testMode'];
    $items = [];
    //error_log(print_r($params['cart'], true));
    foreach ($params['cart']->items as $item) {
        $items[] = [
            'merchantItemId' => $item->id,
            'itemType' => 'Ürün',
            'itemCategory' => 'Ürün',
            'itemName' => $item->name,
            'itemPrice' => $params['amount'],
            'itemQuantity' => (int) $item->qty,
        ];
    }

    $systemUrl = $params['systemurl'];
    $billing = [
        "billingName" => $params['clientdetails']['firstname'] . ' ' . $params['clientdetails']['lastname'],
        "billingCity" => $params['clientdetails']['city'],
        "billingCountry" => 'Türkiye',
        "billingAddress" => $params['clientdetails']['address1'],
    ];

    $shipping = [
        "shippingContactName" => $params['clientdetails']['firstname'] . ' ' . $params['clientdetails']['lastname'],
        "shippingCity" => $params['clientdetails']['city'],
        "shippingCountry" => 'Türkiye',
        "shippingAddress" => $params['clientdetails']['address1'],
    ];

    $cardType = $params['cardtype'];
    $cardNumber = $params['cardnum'];
    $cardExpiry = $params['cardexp'];
    $cardStart = $params['cardstart'];
    $cardIssueNumber = $params['cardissuenum'];
    $cardCvv = $params['cccvv'];

    $card_data = [
        "cardNo" => $cardNumber,
        "cardHolderName" => $params['clientdetails']['firstname'] . ' ' . $params['clientdetails']['lastname'],
        "expDate" => substr($cardExpiry, 0, 2) . '/' . substr($cardExpiry, 2),
        "cvv" => $cardCvv, 
        "installment" => 1,
        "isPreAuth" => "false"
    ];

    $data = [

        // Customer
        'customerName' => $params['clientdetails']['firstname'] . ' ' . $params['clientdetails']['lastname'],
        'customerIpAddress' => teqpaycard_ip(),
        'customerCitizenNo' => 12345678901,
        'customerEmail' => $params['clientdetails']['email'],
        'customerPhone' => $params['clientdetails']['phonenumber'],
        'language' => 'TR',
        // Order
        'price' => number_format(abs($params['amount']), 2, ".", ""),
        'currency' => $params['currency'],

        'conversationId' => $params['invoiceid'] . 'WHMCS' . md5(microtime()),

        // Products
        'products' => $items,

        // Billign
        'billing' => $billing,

        'cardData' => $card_data,

        'paymentMethodId' => "1",
        'moneyTransferData' => [
            'ibanNo' => null,
            'moneyTransferCode' => null

        ],

        // Shipping
        'shipping' => $shipping,

        'callbackUrl' => $systemUrl . 'modules/gateways/callback/teqpay.php',
        
    ];
    
     
    $url = $testMode == 'on' ? 'https://merchant-test.teqpay.com/api/teqpay/payment' : 'https://merchant.teqpay.com/api/teqpay/payment';

    $response = teqpaycard_curl($data, $api_key, $secret_key, $url);
    error_log(json_encode($data));
    if($response['Result'] == true){
        echo ("<script>location.href='" . $response['PaymentData']['PaymentUrl'] . "'</script>");
        exit;
    } else {
        //throw new Exception($response['ResultMessage']);

    }
}

function teqpaycard_ip()
	{
		$ip = '';
		if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
			$ip = $_SERVER['HTTP_CLIENT_IP'];
		} elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
		} else {
			$ip = $_SERVER['REMOTE_ADDR'];
		}

		return $ip;
	}

    function teqpaycard_curl($data, $api_key, $secret_key, $url)
    {
        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_VERBOSE => true,
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => json_encode($data),
            CURLOPT_HTTPHEADER => [
                "Content-Type: application/json",
                "ApiKey: {$api_key}",
                "SecretKey: {$secret_key}",
            ]
        ]);
    
        $response = curl_exec($curl);
        $response = json_decode($response, true);
        curl_close($curl);
    
        return $response;
    }
    
    
